-- luacheck: no unused

---@class lib.anim8
local anim8 = { _VERSION = "anim8 v2.3.1" }

---@class lib.anim8-grid-class
local Grid = {}

---@alias lib.anim8-grid lib.anim8-grid-class|fun(...):love.Quad[]

--[[
- `frameWidth` and `frameHeight` are the dimensions of the animation _frames_ -
  each of the individual "sub-images" that compose the animation. They are
  usually the same size as your character (so if the character is 32x32 pixels,
  `frameWidth` is 32 and so is `frameHeight`)
- `imageWidth` and `imageHeight` are the dimensions of the image where all the
  frames are. In LÖVE, they can be obtained by doing `image:getWidth()` and
  `image:getHeight()`.
- `left` and `top` are optional, and both default to 0. They are "the left and
  top coordinates of the point in the image where you want to put the origin of
  coordinates of the grid". If all the frames in your grid are the same size,
  and the first one's top-left corner is 0,0, you probably won't need to use
  `left` or `top`.
- `border` is also an optional value, and it also defaults to zero. What
  `border` does is allowing you to define "gaps" between your frames in the
  image. For example, imagine that you have frames of 32x32, but they have a
  1-px border around each frame. So the first frame is not at 0,0, but at 1,1
  (because of the border), the second one is at 1,33 (for the extra border) etc.
  You can take this into account and "skip" these borders.
]]
---@param frameWidth integer
---@param frameHeight integer
---@param imageWidth integer
---@param imageHeight integer
---@param left? integer
---@param top? integer
---@param border? integer
---@return lib.anim8-grid
function anim8.newGrid(
  frameWidth,
  frameHeight,
  imageWidth,
  imageHeight,
  left,
  top,
  border
) end

--[[
- Each two numbers are interpreted as quad coordinates in the format
  `(column, row)`. This way, `grid:getFrames(3,4)` will return the frame in
  column 3, row 4 of the grid. There can be more than just two:
  `grid:getFrames(1,1, 1,2, 1,3)` will return the frames in {1,1}, {1,2} and
  {1,3} respectively.
- Using numbers for long rows or columns is tedious - so grids also accept
  strings indicating range plus a row/column index. Differentiating rows and
  columns is based on the order in which the range and index are provided. A row
  can be fetch by calling `grid:getFrames('range', rowNumber)` and a column by
  calling `grid:getFrames(columnNumber, 'range')`. The previous column of 3
  elements, for example, can be also expressed like this:
  `grid:getFrames(1,'1-3')`. Again, there can be more than one string-index pair
  (`grid:getFrames(1,'1-3', '2-4',3)`)
- It's also possible to combine both formats. For example:
  `grid:getFrames(1,4, 1,'1-3')` will get the frame in {1,4} plus the frames 1
  to 3 in column 1
]]
---@return love.Quad[]
function Grid:getFrames(...) end

---@class lib.anim8-animation
local Animation = {}

--[[
- `frames` is an array of frames (Quads in LÖVE argot). You could provide your
  own quad array if you wanted to, but using a grid to get them is very
  convenient.
- `durations` is a number or a table. When it's a number, it represents the
  duration of all frames in the animation. When it's a table, it can represent
  different durations for different frames. You can specify durations for all
  frames individually, like this: `{0.1, 0.5, 0.1}` or you can specify durations
  for ranges of frames: `{['3-5']=0.2}`.
- `onLoop` is an optional parameter which can be a function or a string
  representing one of the animation methods. It does nothing by default. If
  specified, it will be called every time an animation "loops". It will have two
  parameters: the animation instance, and how many loops have been elapsed. The
  most usual value (apart from none) is the string 'pauseAtEnd'. It will make
  the animation loop once and then pause and stop on the last frame.
]]
---@param frames love.Quad[]
---@param durations number
---@param onLoop? function|string
---@return lib.anim8-animation
function anim8.newAnimation(frames, durations, onLoop) end

---Creates a new animation identical to the current one. The only difference is
---that its internal counter is reset to 0 (it's on the first frame).
---@return lib.anim8-animation
function Animation:clone() end

--[[
Flips an animation. This means
that the frames are simply drawn differently, nothing more.

Note that this method does not create a new animation. If you want to create a
new one, use the `clone` method.

This method returns the animation, so you can do things like
`local a = anim8.newAnimation(g(1,'1-10'), 0.1):flipH()` or
`local b = a:clone():flipV()`.

`animation:flipV()`

Flips an animation vertically. The same rules that apply to `flipH` also apply
here.
]]
---@alias lib.anim8-flip fun(): lib.anim8-animation

---@type lib.anim8-flip
Animation.flipH = nil

---@type lib.anim8-flip
Animation.flipV = nil

---Use this inside `love.update(dt)` so that your animation changes frames
---according to the time that has passed.
---@param dt number
function Animation:update(dt) end

---Stops the animation from updating.
function Animation:pause() end

---Moves the animation to a given frame (frames start counting in 1).
---@param position integer
function Animation:gotoFrame(position) end

---Moves the animation to its last frame and then pauses it.
function Animation:pauseAtEnd() end

---Moves the animation to its first frame and then pauses it.
function Animation:pauseAtStart() end

---Unpauses an animation
function Animation:resume() end

--[[
Draws the current frame in the specified coordinates with the right angle,
scale, offset & shearing. These parameters work exactly the same way as in
[love.graphics.draw](https://love2d.org/wiki/love.graphics.draw). The only
difference is that they are properly recalculated when the animation is flipped
horizontally, vertically or both. See `getFrameInfo` below for more details.
]]
---@param image love.Image
---@param x? number
---@param y? number
---@param r? number
---@param sx? number
---@param sy? number
---@param ox? number
---@param oy? number
---@param kx? number
---@param ky? number
function Animation:draw(image, x, y, r, sx, sy, ox, oy, kx, ky) end

--[[
This functions returns the parameters that would be passed to
`love.graphics.draw` when drawing this animation:
`frame, x, y, r, sx, sy, ox, oy, kx, ky`.

- `frame` is the currently active frame for the animation (usually a quad
  produced by a grid)
- `x,y` are the same coordinates passed as parameter to `getFrame` (there are no
  changes)
- `r` is the same angle passed to `getFrame`, with no changes unless it is
  `nil`, in which case it becomes 0.
- `sx,sy` are the scale values, with their sign changed if the animation is
  flipped vertically or horizontally
- `ox,oy` are the offset values, with the width or height properly substracted
  if the animation is flipped. 0 is used as a initial value for these
  calculations if nil was passed.
- `kx,ky` are the shearing factors, changed depending on the flip status.

The `getFrame` method can be used when working with
[spriteBatches](https://love2d.org/wiki/SpriteBatch). Here's how it can be used
for adding & setting the corresponding quad in a spritebatch:

```lua
local id = spriteBatch:add(animation:getFrameInfo(x,y,r,sx,sy,ox,oy,kx,ky))

...

spriteBatch:set(id, animation:getFrameInfo(x,y,r,sx,sy,ox,oy,kx,ky))
```
]]
---@param x? number
---@param y? number
---@param r? number
---@param sx? number
---@param sy? number
---@param ox? number
---@param oy? number
---@param kx? number
---@param ky? number
---@return love.Quad? frame
---@return number? x
---@return number? y
---@return number? r
---@return number? sx
---@return number? sy
---@return number? ox
---@return number? oy
---@return number? kx
---@return number? ky
function Animation:getFrameInfo(x, y, r, sx, sy, ox, oy, kx, ky) end

---Returns the width and height of the current frame of the animation.
---This method assumes the frames passed to the animation are all quads
---(like the ones created by a grid).
---@return number width
---@return number height
function Animation:getDimensions() end
